import requests
import responses

from fake_vault import FakeVault


class TestFakeVault(object):
    def setup(self):
        self.fake_vault = FakeVault()
        self.fake_vault.setup()
        self.fake_vault_url = 'https://test-vault-instance.net'

    @responses.activate
    def test_get_wrapped_token(self):
        self.fake_vault.add_handlers(responses, self.fake_vault_url)
        headers = {
            'X-Vault-Token': '00000000-0000-0000-0000-000000000000',
            'X-Vault-Wrap-TTL': '20s'
        }
        payload = {
            'policies': ['default', 'django-consumer'],
            'ttl': '1h',
            'renewable': True
        }

        resp = requests.post(self.fake_vault_url + '/v1/auth/token/create',
                             headers=headers, json=payload)

        assert resp.json() == {
            'request_id': '',
            'lease_id': '',
            'renewable': False,
            'lease_duration': 0,
            'data': None,
            'wrap_info': {
                'token': '10000000-1000-1000-1000-100000000000',
                'ttl': 200,
                'creation_time': '2016-10-13T15:32:05.6789703Z'
            },
            'warnings': None,
            'auth': None
        }

    @responses.activate
    def test_unwrap_client_token(self):
        self.fake_vault.add_handlers(responses, self.fake_vault_url)
        headers = {
            'X-Vault-Token': '10000000-1000-1000-1000-100000000000'
        }

        resp = requests.post(self.fake_vault_url + '/v1/sys/wrapping/unwrap',
                             headers=headers)
        assert resp.json() == {
                'request_id': '',
                'lease_id': '',
                'lease_duration': 0,
                'renewable': False,
                'auth': {
                    'client_token': '00000000-0000-0000-0000-000000000001',
                    'accessor': '',
                    'policies': ['default', 'gatekeeper'],
                    'metadata': None,
                    'lease_duration': 2764800,
                    'renewable': True
                },
                'warnings': None
            }

    @responses.activate
    def test_authenticated(self):
        self.fake_vault.add_handlers(responses, self.fake_vault_url)
        headers = {
            'X-Vault-Token': '00000000-0000-0000-0000-000000000001'
        }

        resp = requests.get(self.fake_vault_url + '/v1/auth/token/lookup-self',
                            headers=headers)
        assert resp.json() == {
            'data': {
                'policies': ['default', 'django-consumer']
            }
        }

    @responses.activate
    def test_get_psql_creds(self):
        self.fake_vault.add_handlers(responses, self.fake_vault_url)
        headers = {
            'X-Vault-Token': '00000000-0000-0000-0000-000000000001'
        }

        resp = requests.get(self.fake_vault_url
                            + '/v1/database/creds/'
                              'postgresql_myschema_readonly',
                            headers=headers)

        assert resp.json() == {
            'lease_id':
                'database/creds/postgresql_myschema_readonly/lease-id1',
            'lease_duration': 100,
            'renewable': True,
            'data': {
                'username': 'testuser1',
                'password': 'testpass1'
            }
        }

    @responses.activate
    def test_renew_lease(self):
        self.fake_vault.add_handlers(responses, self.fake_vault_url)
        headers = {
            'X-Vault-Token': '00000000-0000-0000-0000-000000000001'
        }

        payload = {
            'lease_id':
                'database/creds/postgresql_myschema_readonly/lease-id1',
            'increment': 30
        }
        resp = requests.put(self.fake_vault_url + '/v1/sys/leases/renew',
                            headers=headers,
                            json=payload)

        assert resp.json() == {
            'lease_id':
                'database/creds/postgresql_myschema_readonly/lease-id1',
            'lease_duration': 30,
            'renewable': True
        }

    @responses.activate
    def test_renew_expired_lease(self):
        self.fake_vault.add_handlers(responses, self.fake_vault_url)
        self.fake_vault.expire_lease()
        headers = {
            'X-Vault-Token': '00000000-0000-0000-0000-000000000001'
        }

        payload = {
            'lease_id':
                'database/creds/postgresql/'
                'postgresql_myschema_readonly/lease-id1',
            'increment': 30
        }
        resp = requests.put(self.fake_vault_url + '/v1/sys/leases/renew',
                            headers=headers,
                            json=payload)

        # TODO check Vault response when expired
        assert resp.status_code == 403

    @responses.activate
    def test_renew_self(self):
        self.fake_vault.add_handlers(responses, self.fake_vault_url)
        headers = {
            'X-Vault-Token': '00000000-0000-0000-0000-000000000001'
        }

        payload = {
            'increment': 35
        }
        resp = requests.post(self.fake_vault_url + '/v1/auth/token/renew-self',
                             headers=headers,
                             json=payload)
        assert resp.json() == {
          "auth": {
            "client_token": '00000000-0000-0000-0000-000000000001',
            "policies": ['default', 'django-consumer'],
            "lease_duration": 35,
            "renewable": True,
          }
        }

    @responses.activate
    def test_lookup_self(self):
        self.fake_vault.add_handlers(responses, self.fake_vault_url)
        headers = {
            'X-Vault-Token': '00000000-0000-0000-0000-000000000001'
        }
        resp = requests.get(self.fake_vault_url + '/v1/auth/token/lookup-self',
                            headers=headers)
        assert resp.json() == {
            'data': {
                'policies': ['default', 'django-consumer']
            }
        }

    @responses.activate
    def test_revoke_self(self):
        self.fake_vault.add_handlers(responses, self.fake_vault_url)
        headers = {
            'X-Vault-Token': '00000000-0000-0000-0000-000000000001'
        }
        resp = requests.put(self.fake_vault_url + '/v1/auth/token/revoke-self',
                            headers=headers)
        assert resp.status_code == 200

        resp = requests.get(self.fake_vault_url + '/v1/auth/token/lookup-self',
                            headers=headers)
        assert resp.status_code == 403



import logging
import json
import os
import sys
import shlex
import subprocess32 as subprocess
from subprocess32 import TimeoutExpired
from configparser import ConfigParser
import secret
import hvac
import requests


def get_mesos_taskid(env=os.environ):
    taskid = env['MESOS_TASK_ID']
    if taskid is None:
        raise KeyError('Could not retrieve Mesos task ID.')
    return taskid


def get_vaultkeeper_cfg(env=os.environ):
    path = env['VAULTKEEPER_CONFIG']
    if path is None:
        raise KeyError('Could not retrieve Vaultkeeper config path.')
    return path


def get_secrets_cfg(env=os.environ):
    path = env['SECRETS_CONFIG']
    if path is None:
        raise KeyError('Could not retrieve Secrets configuration path.')
    return path


def get_marathon_appname(env=os.environ):
    appname = env['MARATHON_APP_ID']
    if appname is None:
        raise KeyError('Could not retrieve Marathon app name.')
    return appname


def get_vault_addr(env=os.environ):
    vault_addr = env['VAULT_ADDR']
    if vault_addr is None:
        raise KeyError('Could not retrieve Vault address.')
    return vault_addr


def get_gatekeeper_addr(env=os.environ):
    gatekeeper_addr = env['GATEKEEPER_ADDR']
    if gatekeeper_addr is None:
        raise KeyError('Could not retrieve Gatekeeper address.')
    return gatekeeper_addr


class Vaultkeeper(object):
    logger = logging.getLogger(__name__)

    def __init__(self,
                 configs=None, secrets=None,
                 taskid=None, appname=None,
                 vault_addr=None,
                 gatekeeper_addr=None):
        """
        Create the Vaultkeeper service.

        :param configs: A ConfigParser object.
        :param secrets: A nested dictionary of Secret objects.
        :param taskid: The Mesos task ID for this process' context.
        :param appname: The Marathon app name for this process' context.
        :param vault_addr: The address for the Vault server.
        :param gatekeeper_addr: The address for the Gatekeeper server.
        """
        self.configs = configs
        self.secrets = secrets
        self.taskid = taskid
        self.appname = appname
        self.vault_addr = vault_addr
        self.gatekeeper_addr = gatekeeper_addr
        self.app = None

    def setup(self):
        self.vault_client = hvac.Client(url=self.vault_addr)

    def get_wrapped_token(self):
        payload = {'task_id': self.taskid}
        r = requests.post(self.gatekeeper_addr + '/token',
                          json=payload)
        response = r.json()
        if response['ok']:
            self.wrapped_token = response['token']
            return self.wrapped_token
        raise RuntimeError('The service encountered an error '
                           + 'retrieving its wrapped token '
                           + 'from Gatekeeper: '
                           + response.text)

    def unwrap_token(self, wrapped_token):
        self.vault_secret = secret.UnwrappedToken('vault_token', 'token')
        response = self.vault_client.unwrap(wrapped_token)
        self.vault_secret.add_secret(response)
        self.vault_client.token = self.vault_secret.token_value
        if not self.vault_client.is_authenticated():
            raise RuntimeError('The service could not authenticate'
                               + 'to Vault with the unwrapped token.')
        return self.vault_client.token

    def write_credentials(self):
        data = secret.printable_secrets(self.secrets)
        with open(self.configs.credential_path, 'w') as outfile:
            json.dump(data, outfile)

    def get_cred(self, vault_path):
        if not self.vault_client.is_authenticated():
            raise RuntimeError('The service could not authenticate '
                               + 'to Vault to retrieve credentials.')
        return self.vault_client.read(vault_path)

    def get_creds(self):
        for cred in self.secrets.itervalues():
            response = self.get_cred(cred.vault_path)
            cred.add_secret(response)

    def renew_token(self, ttl):
        result = self.vault_client.renew_token(increment=ttl)
        self.vault_secret.update_ttl(ttl)
        return result

    def renew_lease(self, s):
        assert self.vault_client.is_authenticated()
        result = self.vault_client.renew_secret(s.lease_id,
                                                s.lease_duration)
        s.update_lease(s.lease_id, s.lease_duration)
        return result

    def renew_all(self):
        for entry in self.secrets.itervalues():
            if entry.renewable:
                self.renew_lease(entry)

    def cleanup(self):
        self.vault_client.revoke_self_token()

    def start_subprocess(self):
        self.get_wrapped_token()
        self.unwrap_token(self.wrapped_token)
        self.logger.info('Written credentials to '
                         + self.configs.credential_path)
        self.get_creds()
        self.write_credentials()
        args = shlex.split(self.configs.entry_cmd.encode(
            'utf-8', errors='ignore'))
        self.app = subprocess.Popen(args,
                                    shell=False
                                    )

    def watch_and_renew(self):
        while True:
            try:
                self.app.wait(timeout=self.configs.refresh_interval)
            except TimeoutExpired:
                self.logger.info('Renewing leases...')
                self.renew_token(self.vault_secret.lease_duration)
                self.renew_all()
            else:
                self.cleanup()
                return self.app.returncode

    def run(self):
        self.start_subprocess()
        return self.watch_and_renew()


def main():
    config = get_vaultkeeper_cfg()
    secrets = get_secrets_cfg()
    taskid = get_mesos_taskid()
    appname = get_marathon_appname()
    vault_addr = get_vault_addr()
    gatekeeper_addr = get_gatekeeper_addr()

    configs = ConfigParser(config_path=config)
    configs.load_configs()

    required_secrets = secret.parse_secret_file(secrets)

    vaultkeeper = Vaultkeeper(configs=configs,
                              secrets=required_secrets,
                              taskid=taskid,
                              appname=appname,
                              vault_addr=vault_addr,
                              gatekeeper_addr=gatekeeper_addr
                              )
    vaultkeeper.setup()
    returncode = vaultkeeper.run()
    sys.exit(returncode)


if __name__ == '__main__':
    main()



from fake_vault import FakeVault
import json


class FakeGatekeeper(object):
    def setup(self):
        self.fake_vault = FakeVault()
        self.fake_vault.setup()
        self.vault_token = '00000000-0000-0000-0000-000000000000'
        self.policies = {
            'purple-rain-486': {
                'policies': ['default', 'django-consumer'],
                'ttl': 300
            }
        }

    def reload_policies(self):
        pass

    def task_authorised(self, taskid):
        for policies in self.policies:
            if taskid.startswith(policies):
                return True
        return False

    def get_token(self, request):
        taskid = json.loads(request.body)['task_id']
        if not self.task_authorised(taskid):
            return (401, {}, {})

        headers = {'content-type': 'application/json'}
        body = {
            'ok': True,
            'token': '10000000-1000-1000-1000-100000000000'
        }
        return (200, headers, json.dumps(body))

    def add_handlers(self, responses, fake_gatekeeper_url):
        responses.add_callback(responses.POST,
                               fake_gatekeeper_url + '/token',
                               callback=self.get_token,
                               content_type='application/json')



import json


class ConfigParser(object):
    def __init__(self, config_path=None):
        self.config_path = config_path
        self.entry_script = None
        self.credential_path = None
        self.refresh_interval = None
        self.renewal_grace = None

    def load_data(self, data):
        self.entry_cmd = data['entry_cmd']
        self.credential_path = data['credential_path']
        self.refresh_interval = data['refresh_interval']
        self.renewal_grace = data['renewal_grace']

    def load_configs(self):
        with open(self.config_path) as agent_config:
            data = json.load(agent_config)
        self.load_data(data)



from fake_gatekeeper import FakeGatekeeper
import responses
import requests


class TestFakeGatekeeper(object):
    def setup(self):
        self.fake_gatekeeper = FakeGatekeeper()
        self.fake_gatekeeper.setup()
        self.fake_gatekeeper_url = 'https://test-gatekeeper-instance.net'

    @responses.activate
    def test_get_token(self):
        taskid = 'purple-rain-486-ab24134bed3423f124937'

        responses.add_callback(responses.POST,
                               self.fake_gatekeeper_url + '/token',
                               callback=self.fake_gatekeeper.get_token,
                               content_type='application/json')

        payload = {
            'task_id': taskid
        }

        resp = requests.post(self.fake_gatekeeper_url + '/token',
                             json=payload)

        assert resp.json() == {
            'ok': True,
            'token': '10000000-1000-1000-1000-100000000000'
        }



# -*- coding: utf-8 -*-

from setuptools import setup, find_packages


with open('README.rst') as f:
    readme = f.read()

install_requires = [
                    'hvac>=0.2.17',
                    'requests',
                    'subprocess32'
]

extras_require = {
        'test': [
            'pytest>=3.0.0',
            'responses',
            'pytest-cov',
        ]
}

dependency_links = [
    ('https://github.com/mracter/hvac/archive/'
        'b0811f0446760125baefea6383711035104e6df1.zip'
        '#egg=hvac-0.2.17+git.b817da4'),
]

setup(
    name='vaultkeeper',
    version='0.0.1',
    description=('An agent that works with '
                 'vault-gatekeeper-mesos to '
                 'fetch and renew Vault credentials.'),
    long_description=readme,
    install_requires=install_requires,
    extras_require=extras_require,
    author='mracter',
    author_email='mary@praekelt.org',
    url='https://github.com/praekeltfoundation/vaultkeeper',
    license='BSD',
    packages=find_packages(),
    dependency_links=dependency_links,
    entry_points={
        'console_scripts': [
            'vaultkeeper = vaultkeeper.vaultkeeper:main'
        ]
    },
)
